"""
Quick demo of calling NRL MSISE-00 using f2py from Python
Michael Hirsch, Ph.D.

Original fortran code from
http://nssdcftp.gsfc.nasa.gov/models/atmospheric/msis/nrlmsise00/
"""
from datetime import datetime
import xarray
import numpy as np
#
from sciencedates import datetime2gtd
from gridaurora import readmonthlyApF107
#
import gtd7
#
MASS = 48 # compute all parameters
TSELECOPS = np.array([1,1,1,1,1,1,1,1,-1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1],float)
species = ['He','O','N2','O2','Ar','Total','H','N','AnomalousO']
ttypes = ['Texo','Tn']
first=True

def run(time, altkm:float, glat:float, glon:float) -> xarray.Dataset:
    """
    loops the rungtd1d function below. Figure it's easier to troubleshoot in Python than Fortran.
    """
    glat = np.atleast_2d(glat); glon=np.atleast_2d(glon) #has to be here
#%% altitude 1-D
    if glat.size==1 and glon.size==1 and time.size==1:
        atmos = rungtd1d(time, altkm,glat,glon)
#%% lat/lon grid at 1 altitude
    else:
       atmos = loopalt_gtd(time, glat,glon,altkm)

    return atmos


def loopalt_gtd(time, glat,glon,altkm) -> xarray.Dataset:
    time = np.atleast_1d(time) #keep for code reuse

    atmos = None

    for k,t in enumerate(time):
        print(f'computing {t}')
        for i in range(glat.shape[0]):
            for j in range(glat.shape[1]):
                if atmos is None:
                    atmos = rungtd1d(t, altkm, glat[i,j], glon[i,j])
                    attrs = atmos.attrs
                else:
                    #atmos = xarray.concat((atmos, rungtd1d(t, altkm, glat[i,j], glon[i,j])),
                    #                      data_vars='minimal',coords='minimal',dim='lon')
                    atmos = xarray.merge((atmos,rungtd1d(t, altkm, glat[i,j], glon[i,j])))

    atmos.attrs = attrs

    return atmos


def rungtd1d(time, altkm:float, glat:float, glon:float) -> xarray.Dataset:
    """
    This is the "atomic" function looped by other functions
    """
    # %% get solar parameters for date
    f107Ap = readmonthlyApF107(time)
    f107a = f107Ap['f107s'].item()
    f107  = f107Ap['f107o'].item()
    Ap    = (f107Ap['Apo'].item(),)*7
#%%
    altkm= np.atleast_1d(altkm)
    glon = np.atleast_1d(glon).squeeze()
    glat = np.atleast_1d(glat).squeeze()

    assert isinstance(time,(np.datetime64,datetime,str)),'if you have multiple times, for loop over them'

# don't check ap, too complicated
    assert isinstance(MASS,(float,int))
    assert len(TSELECOPS)==25
#%%
    Ap = np.atleast_1d(Ap)
    if Ap.size==1:
        Ap = np.repeat(Ap,7)

    gtd7.tselec(TSELECOPS) #like the msis_driver example

    iyd,utsec,stl = datetime2gtd(time, glon)
    altkm         = np.atleast_1d(altkm)

    gtd7.meters(1) # makes output in m^-3 and kg/m^-3

    dens = np.empty((altkm.size, len(species)))
    temp = np.empty((altkm.size, len(ttypes)))
    for i,a in enumerate(altkm):
        dens[i,:], temp[i,:] = gtd7.gtd7(iyd, utsec, a, glat,glon, stl, f107a, f107, Ap,MASS)


    dsf = {k: (('time','alt_km','lat','lon'),v[None,:,None,None]) for (k,v) in zip(species, dens.T)}
    dsf.update({'Tn':  (('time','alt_km','lat','lon'), temp[:,1][None,:,None,None]),
                'Texo':(('time','alt_km','lat','lon'), temp[:,0][None,:,None,None])})

    atmos = xarray.Dataset(dsf,
                           coords={'time':[time], 'alt_km':altkm, 'lat':[glat],'lon':[glon],},
                           attrs={'Ap':Ap,'f107':f107,'f107a':f107a,
                                  'species':species})

    return atmos



from pathlib import Path
import xarray
import numpy as np
from astropy.time import Time
from astropy.coordinates import get_sun,EarthLocation, AltAz
from matplotlib.pyplot import figure, close
from matplotlib.ticker import ScalarFormatter
#
from pymap3d import aer2geodetic

def plotgtd(atmos:xarray.Dataset, rodir=None):
#
    if rodir:
        rodir = Path(rodir).expanduser()

    sfmt = ScalarFormatter(useMathText=True) #for 10^3 instead of 1e3
    sfmt.set_powerlimits((-2, 2))
    sfmt.set_scientific(True)
    sfmt.set_useOffset(False)

    if atmos['N2'].squeeze().ndim==1: #altitude 1-D
        plot1d(atmos)
    elif atmos['N2'].ndim==4: #lat/lon grid
#%% sun lat/lon
        time = Time(str(atmos.time[0].values))
        obs = EarthLocation(0,0) # geodetic lat,lon = 0,0 arbitrary
        sun = get_sun(time=time)
        aaf = AltAz(obstime=time,location=obs)
        sloc = sun.transform_to(aaf)
        slat,slon = aer2geodetic(sloc.az.value, sloc.alt.value, sloc.distance.value,0,0,0)[:2]
        slat = np.atleast_1d(slat); slon = np.atleast_1d(slon)
#%% tableau
        for i,t in enumerate(atmos.time):
            fg = figure(figsize=(8,8))
            ax = fg.subplots(4,2,sharex=True)
            fg.suptitle(str(t.values)[:-13] +
                        f' alt.(km) {atmos.alt_km[0].item()}\n'
                        f'Ap={atmos.Ap[0]}  F10.7={atmos.f107.item()}')
            ax=ax.ravel()

            j=0
            for s in atmos.species:
                if s == 'Total':
                    continue

                a = ax[j]

                hi = a.imshow(atmos[s][i][0], aspect='auto',
                        interpolation='none',cmap='viridis',
                         extent=(atmos.lon[0], atmos.lon[-1], atmos.lat[0], atmos.lat[-1]))

                fg.colorbar(hi,ax=a, format=sfmt)
    # %% sun icon moving
                a.plot(slon[i],slat[i],linestyle='none',
                                    marker='o',markersize=5,color='w')

                a.set_title(f'Density: {s}')
                a.set_xlim(-180,180)
                a.set_ylim(-90,90)
                a.autoscale(False)
                j+=1

            for k in range(0,6+2,2):
                ax[k].set_ylabel('latitude (deg)')
            for k in (6,7):
                ax[k].set_xlabel('longitude (deg)')

            if rodir:
                ofn = rodir / f'{atmos.alt_km[0].item():.1f}_{i:03d}.png'
                print('writing',ofn)
                fg.savefig(ofn, dpi=100, bbox_inches='tight')
                close()

    else:
        print(atmos)


def plot1d(atmos:xarray.Dataset):

    footer = f'\n({atmos.lat.item()},{atmos.lon.item()})  Ap {atmos.Ap[0]}  F10.7 {atmos.f107}'

    z = atmos.alt_km.values
    ax = figure().gca()
    for s in atmos.species:
        if s == 'Total':
            continue
        ax.semilogx(atmos[s].squeeze(), z, label=s)
    ax.legend(loc='best')
    ax.set_xlim(left=1e3)
    ax.set_ylabel('altitude [km]')
    ax.set_xlabel('density [m$^{-3}$]')
    ax.grid(True)
    ax.set_title('Number Density from MSISE-00' + footer)

    ax = figure().gca()
    ax.semilogx(atmos['Total'].squeeze(), z)
    ax.set_xlabel('Total Mass Density [kg m$^{-3}$]')
    ax.set_ylabel('altitude [km]')
    ax.grid(True)
    ax.set_title('Total Mass Density from MSISE-00'+footer)

    ax = figure().gca()
    ax.plot(atmos['Tn'].squeeze(), z)
    ax.set_xlabel('Temperature')
    ax.set_ylabel('altitude [km]')
    ax.grid(True)
    ax.set_title('Temperature from MSISE-00' + footer)



#!/usr/bin/env python
install_requires= ['python-dateutil','pytz','numpy','xarray',
                   'sciencedates','gridaurora']
tests_require = ['pytest','nose','coveralls']
# %%
from setuptools import find_packages
from numpy.distutils.core import setup,Extension

setup(name='msise00',
      packages=find_packages(),
      description='Python API for Fortran MSISE-00 neutral atmosphere model.',
      author='Michael Hirsch, Ph.D.',
      version='1.1.1a',
      url='https://github.com/scivision/msise00',
      classifiers=[
      'Intended Audience :: Science/Research',
      'Development Status :: 4 - Beta',
      'License :: OSI Approved :: MIT License',
      'Topic :: Scientific/Engineering :: Atmospheric Science',
      'Programming Language :: Python',
      'Programming Language :: Python :: 3.6',
      ],
      ext_modules=[Extension(name='gtd7',
                sources=['fortran/nrlmsise00_sub.for'],
                f2py_options=['--quiet'])],
      install_requires=install_requires,
      tests_require=tests_require,
      python_requires='>=3.6',
      extras_require={'plot':['matplotlib','seaborn'],
                      'io':['astropy','pymap3d'],
                      'tests':tests_require},
      script=['RunMSIS.py'],
      include_package_data=True,
	  )

